@@ -50,9 +50,36 @@ impl<'cmd> Parser<'cmd> {
         &mut self,
         matcher: &mut ArgMatcher,
         raw_args: &mut clap_lex::RawArgs,
-        mut args_cursor: clap_lex::ArgCursor,
+        args_cursor: clap_lex::ArgCursor,
     ) -> ClapResult<()> {
         debug!("Parser::get_matches_with");
+
+        ok!(self.parse(matcher, raw_args, args_cursor).map_err(|err| {
+            if self.cmd.is_ignore_errors_set() {
+                #[cfg(feature = "env")]
+                let _ = self.add_env(matcher);
+                let _ = self.add_defaults(matcher);
+            }
+            err
+        }));
+        ok!(self.resolve_pending(matcher));
+
+        #[cfg(feature = "env")]
+        ok!(self.add_env(matcher));
+        ok!(self.add_defaults(matcher));
+
+        Validator::new(self.cmd).validate(matcher)
+    }
+
+    // The actual parsing function
+    #[allow(clippy::cognitive_complexity)]
+    pub(crate) fn parse(
+        &mut self,
+        matcher: &mut ArgMatcher,
+        raw_args: &mut clap_lex::RawArgs,
+        mut args_cursor: clap_lex::ArgCursor,
+    ) -> ClapResult<()> {
+        debug!("Parser::parse");
         // Verify all positional assertions pass
 
         let mut subcmd_name: Option<String> = None;
@@ -436,11 +463,7 @@ impl<'cmd> Parser<'cmd> {
                     matches: sc_m.into_inner(),
                 });
 
-                ok!(self.resolve_pending(matcher));
-                #[cfg(feature = "env")]
-                ok!(self.add_env(matcher));
-                ok!(self.add_defaults(matcher));
-                return Validator::new(self.cmd).validate(parse_state, matcher);
+                return Ok(());
             } else {
                 // Start error processing
                 let _ = self.resolve_pending(matcher);
@@ -474,11 +497,7 @@ impl<'cmd> Parser<'cmd> {
             ok!(self.parse_subcommand(&sc_name, matcher, raw_args, args_cursor, keep_state));
         }
 
-        ok!(self.resolve_pending(matcher));
-        #[cfg(feature = "env")]
-        ok!(self.add_env(matcher));
-        ok!(self.add_defaults(matcher));
-        Validator::new(self.cmd).validate(parse_state, matcher)
+        Ok(())
     }
 
     fn match_arg_error(
@@ -514,7 +533,7 @@ impl<'cmd> Parser<'cmd> {
                     arg_os.display().to_string(),
                     matcher
                         .arg_ids()
-                        .map(|id| self.cmd.find(id).unwrap().to_string())
+                        .filter_map(|id| self.cmd.find(id).map(|a| a.to_string()))
                         .collect(),
                     Usage::new(self.cmd).create_usage_with_title(&[]),
                 );
@@ -677,7 +696,7 @@ impl<'cmd> Parser<'cmd> {
             debug!("Parser::is_new_arg: --<something> found");
             true
         } else if next.is_short() {
-            // If this is a short flag, this is a new arg. But a singe '-' by
+            // If this is a short flag, this is a new arg. But a single '-' by
             // itself is a value and typically means "stdin" on unix systems.
             debug!("Parser::is_new_arg: -<something> found");
             true
@@ -745,7 +764,7 @@ impl<'cmd> Parser<'cmd> {
         // maybe here lifetime should be 'a
         debug!("Parser::parse_long_arg");
 
-        #[allow(clippy::blocks_in_if_conditions)]
+        #[allow(clippy::blocks_in_conditions)]
         if matches!(parse_state, ParseState::Opt(opt) | ParseState::Pos(opt) if
             self.cmd[opt].is_allow_hyphen_values_set())
         {
@@ -863,7 +882,7 @@ impl<'cmd> Parser<'cmd> {
     ) -> ClapResult<ParseResult> {
         debug!("Parser::parse_short_arg: short_arg={short_arg:?}");
 
-        #[allow(clippy::blocks_in_if_conditions)]
+        #[allow(clippy::blocks_in_conditions)]
         if matches!(parse_state, ParseState::Opt(opt) | ParseState::Pos(opt)
                 if self.cmd[opt].is_allow_hyphen_values_set() || (self.cmd[opt].is_allow_negative_numbers_set() && short_arg.is_negative_number()))
         {
@@ -1165,7 +1184,7 @@ impl<'cmd> Parser<'cmd> {
                         split_raw_vals.extend(raw_val.split(val_delim).map(|x| x.to_owned()));
                     }
                 }
-                raw_vals = split_raw_vals
+                raw_vals = split_raw_vals;
             }
         }
 
@@ -1570,10 +1589,16 @@ impl<'cmd> Parser<'cmd> {
             .collect();
 
         // `did_you_mean` is a lot more likely and should cause us to skip the `--` suggestion
+        // with the one exception being that the CLI is trying to capture arguments
         //
         // In theory, this is only called for `--long`s, so we don't need to check
-        let suggested_trailing_arg =
-            did_you_mean.is_none() && !trailing_values && self.cmd.has_positionals();
+        let suggested_trailing_arg = (did_you_mean.is_none()
+            || self
+                .cmd
+                .get_positionals()
+                .any(|arg| arg.is_last_set() || arg.is_trailing_var_arg_set()))
+            && !trailing_values
+            && self.cmd.has_positionals();
         ClapError::unknown_argument(
             self.cmd,
             format!("--{arg}"),
